@@ -33,6 +33,7 @@
         x.names <- x@.expr
     }
     is.factor <- x@.is.factor
+    factor.ref <- x@.factor.ref
     factor.suffix <- x@.factor.suffix
     x.col.data_type <- x@.col.data_type
     x.col.udt_name <- x@.col.udt_name
@@ -49,6 +50,7 @@
             x.col.data_type <- c(x.col.data_type, value@.col.data_type[i])
             x.col.udt_name <- c(x.col.udt_name, value@.col.udt_name[i])
             is.factor <- c(is.factor, value@.is.factor[i])
+            factor.ref <- c(factor.ref, value@.factor.ref[i])
             factor.suffix <- c(factor.suffix, value@.factor.suffix[i])
         } else {
             if (is.null(case))
@@ -67,6 +69,7 @@
             x.col.data_type[idx] <- value@.col.data_type[i]
             x.col.udt_name[idx] <- value@.col.udt_name[i]
             is.factor[idx] <- value@.is.factor[i]
+            factor.ref[idx] <- value@.factor.ref[i]
             factor.suffix[idx] <- value@.factor.suffix[i]
         }
     }
@@ -102,6 +105,7 @@
         .col.data_type = x.col.data_type,
         .col.udt_name = x.col.udt_name,
         .is.factor = is.factor,
+        .factor.ref = factor.ref,
         .factor.suffix = factor.suffix,
         .sort = sort,
         .dist.by = x@.dist.by)
@@ -115,6 +119,7 @@
     if (is(x, "db.data.frame")) x <- x[,]
     x.names <- x@.expr
     is.factor <- x@.is.factor
+    factor.ref <- x@.factor.ref
     factor.suffix <- x@.factor.suffix
     x.col.data_type <- x@.col.data_type
     x.col.udt_name <- x@.col.udt_name
@@ -132,6 +137,7 @@
             x.col.data_type <- c(x.col.data_type, type)
             x.col.udt_name <- c(x.col.udt_name, udt)
             is.factor <- c(is.factor, FALSE)
+            factor.ref <- c(factor.ref, as.character(NA))
             factor.suffix <- c(factor.suffix, "")
         } else {
             if (is.null(case))
@@ -152,6 +158,7 @@
             x.col.data_type[idx] <- type
             x.col.udt_name[idx] <- udt
             is.factor[idx] <- FALSE
+            factor.ref[idx] <- as.character(NA)
             factor.suffix[idx] <- ""
         }
     }
@@ -196,6 +203,7 @@
         .col.data_type = x.col.data_type,
         .col.udt_name = x.col.udt_name,
         .is.factor = is.factor,
+        .factor.ref = factor.ref,
         .factor.suffix = factor.suffix,
         .sort = sort,
         .dist.by = x@.dist.by)
@@ -370,246 +378,255 @@ setMethod (
 
 ## -----------------------------------------------------------------------
 
-setMethod (
-    "[<-",
-    signature (x = "db.obj", value = "db.Rquery"),
-    function (x, i, j, value) {
-        ## n <- length(sys.calls()[[1]]) - 1
-        n <- nargs()
-        if (is(value, "db.Rquery") && value@.is.agg) {
-            value <- as.numeric(lookat(value))
-            if (missing(i) && missing(j)) {
-                x[,] <- value
-                return (x)
-            }
-            if (missing(i)) {
-                x[,j] <- value
-                return (x)
-            }
-            if (missing(j)) {
-                x[i,] <- value
-                return (x)
-            }
-            x[i,j] <- value
+`[<-.db.obj` <- function(x, i, j, value)
+{
+    has.i <- !missing(i)
+    has.j <- !missing(j)
+
+    if (length(x@.col.name) == 1 && x@.col.data_type == "array" &&
+        value@.is.factor)
+        stop("Cannot set an array to be a factor!")
+
+    if (is(value, "db.Rquery")) func <- "dbRquery"
+    else if (is(value, "character")) func <- "character"
+    else if (is(value, "integer")) func <- "integer"
+    else if (is(value, "numeric")) func <- "numeric"
+    else if (is(value, "logical")) func <- "logical"
+    else stop("value type is not supported!")
+    if (has.i && has.j)
+        eval(parse(text = paste(".square.bracket.", func, "(x=x, i=i, j=j, value=value)", sep = "")))
+    else if (has.i)
+        eval(parse(text = paste(".square.bracket.", func, "(x=x, i=i, value=value)", sep = "")))
+    else if (has.j)
+        eval(parse(text = paste(".square.bracket.", func, "(x=x, j=j, value=value)", sep = "")))
+    else
+        eval(parse(text = paste(".square.bracket.", func, "(x=x, value=value)", sep = "")))
+}
+
+## ----------------------------------------------------------------------
+
+.square.bracket.dbRquery <- function(x, i, j, value)
+{
+    n <- nargs()
+    if (is(value, "db.Rquery") && value@.is.agg) {
+        value <- as.numeric(lookat(value))
+        if (missing(i) && missing(j)) {
+            x[,] <- value
             return (x)
         }
-        
-        if (length(x@.col.name) == 1 && x@.col.data_type == "array") {
-            x <- .expand.array(x)
-            if (n == 3)
-                x[i] <- value
+        if (missing(i)) {
+            x[,j] <- value
+            return (x)
+        }
+        if (missing(j)) {
+            x[i,] <- value
+            return (x)
+        }
+        x[i,j] <- value
+        return (x)
+    }
+    
+    if (length(x@.col.name) == 1 && x@.col.data_type == "array") {
+        x <- .expand.array(x)
+        if (n == 3)
+            x[i] <- value
+        else
+            if (missing(i) && missing(j)) x[,] <- value
+            else if (missing(i)) x[,j] <- value
+            else if (missing(j)) x[i,] <- value
+            else x[i,j] <- value
+        return (db.array(x))
+    }
+    
+    if (n == 4) {
+        if (missing(i) && missing(j))
+            .replacement(x, names(x), value)
+        else if (missing(i))
+            .replacement(x, names(x[,j]), value)
+        else {
+            str <- .case.condition(x, i)
+            if (is(x, "db.data.frame") || x@.where == "")
+                where.str <- str
             else
-                if (missing(i) && missing(j)) x[i,j] <- value
-                else if (missing(i)) x[,j] <- value
-                else if (missing(j)) x[i,] <- value
-                else x[i,j] <- value
-            return (db.array(x))
+                where.str <- paste("(", x@.where, ") and (", str, ")", sep = "")
+            if (missing(j))
+                .replacement(x, names(x), value, str, where.str)
+            else
+                .replacement(x, names(x[i,j]), value, str, where.str)
         }
-        
-        if (n == 4) {
-            if (missing(i) && missing(j))
-                .replacement(x, names(x), value)
-            else if (missing(i))
-                .replacement(x, names(x[,j]), value)
-            else {
-                str <- .case.condition(x, i)
-                if (is(x, "db.data.frame") || x@.where == "")
-                    where.str <- str
-                else
-                    where.str <- paste("(", x@.where, ") and (", str, ")", sep = "")
-                if (missing(j))
-                    .replacement(x, names(x), value, str, where.str)
-                else
-                    .replacement(x, names(x[i,j]), value, str, where.str)
-            }
-        } else if (n == 3) {
-            if (is(i, "db.Rquery")) {
-                str <- .case.condition(x, i)
-                .replacement(x, names(x[i]), value, str)
-            } else
-                .replacement(x, names(x[i]), value)
-        }
-    },
-    valueClass = "db.Rquery")
+    } else if (n == 3) {
+        if (is(i, "db.Rquery")) {
+            str <- .case.condition(x, i)
+            .replacement(x, names(x[i]), value, str)
+        } else
+            .replacement(x, names(x[i]), value)
+    }
+}
 
 ## -----------------------------------------------------------------------
 
-setMethod (
-    "[<-",
-    signature (x = "db.obj", value = "character"),
-    function (x, i, j, value) {
-        value <- paste("'", value, "'", sep = "")
-        ## n <- length(sys.calls()[[1]]) - 1
-        n <- nargs()
-        if (length(x@.col.name) == 1 && x@.col.data_type == "array") {
-            x <- .expand.array(x)
-            if (n == 3)
-                x[i] <- value
-            else
-                if (missing(i) && missing(j)) x[i,j] <- value
-                else if (missing(i)) x[,j] <- value
-                else if (missing(j)) x[i,] <- value
-                else x[i,j] <- value
-            return (db.array(x))
-        }
- 
-        if (n == 4) {
-            if (missing(i) && missing(j))
-                .replace.single(x, names(x), value, "text", "text")
-            else if (missing(i))
-                .replace.single(x, names(x[,j]), value, "text", "text")
-            else {
-                str <- .case.condition(x, i)
-                if (missing(j))
-                    if (is(i, "db.Rquery") && all(i@.col.data_type == "boolean"))
-                        .replace.single(x, names(x), value, "integer", "int4",
-                                        i@.expr)
-                    else
-                        .replace.single(x, names(x), value, "text", "text", str)
-                else 
-                    .replace.single(x, names(x[i,j]), value,
-                                    "text", "text", str)
-            }
-        } else if (n == 3) {
-            if (is(i, "db.Rquery") && all(i@.col.data_type == "boolean"))
-                .replace.single(x, names(x), value, "text", "text",
-                                i@.expr)
-            .replace.single(x, names(x[i]), value, "text", "text")
-        }
-    },
-    valueClass = "db.Rquery")
-
-## -----------------------------------------------------------------------
-
-setMethod (
-    "[<-",
-    signature (x = "db.obj", value = "integer"),
-    function (x, i, j, value) {
-        ## n <- length(sys.calls()[[1]]) - 1
-        n <- nargs()
-        if (length(x@.col.name) == 1 && x@.col.data_type == "array") {
-            x <- .expand.array(x)
-            if (n == 3)
-                x[i] <- value
-            else
-                if (missing(i) && missing(j)) x[i,j] <- value
-                else if (missing(i)) x[,j] <- value
-                else if (missing(j)) x[i,] <- value
-                else x[i,j] <- value
-            return (db.array(x))
-        }
-        if (n == 4) {
-            if (missing(i) && missing(j))
-                .replace.single(x, names(x), value, "integer", "int4")
-            else if (missing(i))
-                .replace.single(x, names(x[,j]), value, "integer", "int4")
-            else if (missing(j))
+.square.bracket.character <- function(x, i, j, value)
+{
+    value <- paste("'", value, "'", sep = "")
+    n <- nargs()
+    if (length(x@.col.name) == 1 && x@.col.data_type == "array") {
+        x <- .expand.array(x)
+        if (n == 3)
+            x[i] <- value
+        else
+            if (missing(i) && missing(j)) x[i,j] <- value
+            else if (missing(i)) x[,j] <- value
+            else if (missing(j)) x[i,] <- value
+            else x[i,j] <- value
+        return (db.array(x))
+    }
+    
+    if (n == 4) {
+        if (missing(i) && missing(j))
+            .replace.single(x, names(x), value, "text", "text")
+        else if (missing(i))
+            .replace.single(x, names(x[,j]), value, "text", "text")
+        else {
+            str <- .case.condition(x, i)
+            if (missing(j))
                 if (is(i, "db.Rquery") && all(i@.col.data_type == "boolean"))
                     .replace.single(x, names(x), value, "integer", "int4",
                                     i@.expr)
                 else
-                    .replace.single(x, names(x), value, "integer", "int4",
-                                    .case.condition(x, i))
-            else
-                .replace.single(x, names(x[i,j]), value, "integer",
-                                "int4", .case.condition(x, i))
-        } else if (n == 3) {
+                    .replace.single(x, names(x), value, "text", "text", str)
+            else 
+                .replace.single(x, names(x[i,j]), value,
+                                "text", "text", str)
+        }
+    } else if (n == 3) {
+        if (is(i, "db.Rquery") && all(i@.col.data_type == "boolean"))
+            .replace.single(x, names(x), value, "text", "text",
+                            i@.expr)
+        .replace.single(x, names(x[i]), value, "text", "text")
+    }
+}
+
+## -----------------------------------------------------------------------
+
+.square.bracket.integer <- function(x, i, j, value)
+{
+    n <- nargs()
+    if (length(x@.col.name) == 1 && x@.col.data_type == "array") {
+        x <- .expand.array(x)
+        if (n == 3)
+            x[i] <- value
+        else
+            if (missing(i) && missing(j)) x[i,j] <- value
+            else if (missing(i)) x[,j] <- value
+            else if (missing(j)) x[i,] <- value
+            else x[i,j] <- value
+        return (db.array(x))
+    }
+    if (n == 4) {
+        if (missing(i) && missing(j))
+            .replace.single(x, names(x), value, "integer", "int4")
+        else if (missing(i))
+            .replace.single(x, names(x[,j]), value, "integer", "int4")
+        else if (missing(j))
             if (is(i, "db.Rquery") && all(i@.col.data_type == "boolean"))
                 .replace.single(x, names(x), value, "integer", "int4",
                                 i@.expr)
             else
-                .replace.single(x, names(x[i]), value, "integer", "int4")
-        }
-    },
-    valueClass = "db.Rquery")
+                .replace.single(x, names(x), value, "integer", "int4",
+                                .case.condition(x, i))
+        else
+            .replace.single(x, names(x[i,j]), value, "integer",
+                            "int4", .case.condition(x, i))
+    } else if (n == 3) {
+        if (is(i, "db.Rquery") && all(i@.col.data_type == "boolean"))
+            .replace.single(x, names(x), value, "integer", "int4",
+                            i@.expr)
+        else
+            .replace.single(x, names(x[i]), value, "integer", "int4")
+    }
+}
 
 ## -----------------------------------------------------------------------
 
-setMethod (
-    "[<-",
-    signature (x = "db.obj", value = "numeric"),
-    function (x, i, j, value) {
-        n <- nargs()
-        if (length(x@.col.name) == 1 && x@.col.data_type == "array") {
-            x <- .expand.array(x)
-            if (n == 3)
-                x[i] <- value
-            else
-                if (missing(i) && missing(j)) x[i,j] <- value
-                else if (missing(i)) x[,j] <- value
-                else if (missing(j)) x[i,] <- value
-                else x[i,j] <- value
-            return (db.array(x))
-        }
-        if (n == 4) {
-            if (missing(i) && missing(j)) {
-                .replace.single(x, names(x), value, "double precision",
-                                "float8")
-            } else if (missing(i)) {
-                .replace.single(x, names(x[,j]), value, "double precision",
-                                "float8")
-            } else if (missing(j)) {
-                if (is(i, "db.Rquery") && all(i@.col.data_type == "boolean")) {
-                    .replace.single(x, names(x), value, "double precision",
-                                    "float8", i@.expr)
-                } else
-                    .replace.single(x, names(x), value, "double precision",
-                                    "float8", .case.condition(x, i))
-            } else {
-                .replace.single(x, names(x[i,j]), value, "double precision",
-                                "float8", .case.condition(x, i))
-            }
-        } else if (n == 3) {
-            if (is(i, "db.Rquery") && all(i@.col.data_type == "boolean"))
+.square.bracket.numeric <- function(x, i, j, value)
+{
+    n <- nargs()
+    if (length(x@.col.name) == 1 && x@.col.data_type == "array") {
+        x <- .expand.array(x)
+        if (n == 3)
+            x[i] <- value
+        else
+            if (missing(i) && missing(j)) x[i,j] <- value
+            else if (missing(i)) x[,j] <- value
+            else if (missing(j)) x[i,] <- value
+            else x[i,j] <- value
+        return (db.array(x))
+    }
+    if (n == 4) {
+        if (missing(i) && missing(j)) {
+            .replace.single(x, names(x), value, "double precision",
+                            "float8")
+        } else if (missing(i)) {
+            .replace.single(x, names(x[,j]), value, "double precision",
+                            "float8")
+        } else if (missing(j)) {
+            if (is(i, "db.Rquery") && all(i@.col.data_type == "boolean")) {
                 .replace.single(x, names(x), value, "double precision",
                                 "float8", i@.expr)
-            else
-                .replace.single(x, names(x[i]), value, "double precision",
-                                "float8")
+            } else
+                .replace.single(x, names(x), value, "double precision",
+                                "float8", .case.condition(x, i))
+        } else {
+            .replace.single(x, names(x[i,j]), value, "double precision",
+                            "float8", .case.condition(x, i))
         }
-    },
-    valueClass = "db.Rquery")
+    } else if (n == 3) {
+        if (is(i, "db.Rquery") && all(i@.col.data_type == "boolean"))
+            .replace.single(x, names(x), value, "double precision",
+                            "float8", i@.expr)
+        else
+            .replace.single(x, names(x[i]), value, "double precision",
+                            "float8")
+    }
+}
+
 
 ## -----------------------------------------------------------------------
 
-setMethod (
-    "[<-",
-    signature (x = "db.obj", value = "logical"),
-    function (x, i, j, value) {
-        ## n <- length(sys.calls()[[1]]) - 1
-        n <- nargs()
-        if (length(x@.col.name) == 1 && x@.col.data_type == "array") {
-            x <- .expand.array(x)
-            if (n == 3)
-                x[i] <- value
-            else
-                if (missing(i) && missing(j)) x[i,j] <- value
-                else if (missing(i)) x[,j] <- value
-                else if (missing(j)) x[i,] <- value
-                else x[i,j] <- value
-            return (db.array(x))
-        }
-        if (n == 4) {
-            if (missing(i) && missing(j))
-                .replace.single(x, names(x), value, "boolean", "bool")
-            else if (missing(i))
-                .replace.single(x, names(x[,j]), value, "boolean", "bool")
-            else if (missing(j))
-                if (is(i, "db.Rquery") && all(i@.col.data_type == "boolean"))
-                    .replace.single(x, names(x), value, "boolean", "bool",
-                                    i@.expr)
-                else
-                    .replace.single(x, names(x), value, "boolean", "bool",
-                                    .case.condition(x, i))
-            else
-                .replace.single(x, names(x[i,j]), value, "boolean", "bool",
-                                .case.condition(x, i))
-        } else if (n == 3) {
+.square.bracket.logical <- function(x, i, j, value)
+{
+    n <- nargs()
+    if (length(x@.col.name) == 1 && x@.col.data_type == "array") {
+        x <- .expand.array(x)
+        if (n == 3)
+            x[i] <- value
+        else
+            if (missing(i) && missing(j)) x[i,j] <- value
+            else if (missing(i)) x[,j] <- value
+            else if (missing(j)) x[i,] <- value
+            else x[i,j] <- value
+        return (db.array(x))
+    }
+    if (n == 4) {
+        if (missing(i) && missing(j))
+            .replace.single(x, names(x), value, "boolean", "bool")
+        else if (missing(i))
+            .replace.single(x, names(x[,j]), value, "boolean", "bool")
+        else if (missing(j))
             if (is(i, "db.Rquery") && all(i@.col.data_type == "boolean"))
                 .replace.single(x, names(x), value, "boolean", "bool",
                                 i@.expr)
             else
-                .replace.single(x, names(x[i]), value, "boolean", "bool")
-        }
-    },
-    valueClass = "db.Rquery")
+                .replace.single(x, names(x), value, "boolean", "bool",
+                                .case.condition(x, i))
+        else
+            .replace.single(x, names(x[i,j]), value, "boolean", "bool",
+                            .case.condition(x, i))
+    } else if (n == 3) {
+        if (is(i, "db.Rquery") && all(i@.col.data_type == "boolean"))
+            .replace.single(x, names(x), value, "boolean", "bool",
+                            i@.expr)
+        else
+            .replace.single(x, names(x[i]), value, "boolean", "bool")
+    }
+}
